# pywws - Python software for USB Wireless Weather Stations
# http://github.com/jim-easterbrook/pywws
# Copyright (C) 2018-22  pywws contributors

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

"""Upload data to Weather Underground.

`Weather Underground`_ may be the oldest and best known site gathering
data from amateur weather stations.

* Create account: https://www.wunderground.com/signup
* API: `<https://support.weather.com/s/article/PWS-Upload-Protocol>`_
* Additional dependency: http://docs.python-requests.org/
* Example ``weather.ini`` configuration::

    [underground]
    station = ABCDEFGH1
    password = xxxxxxx
    internal = False

    [logged]
    services = ['underground', 'metoffice']

    [live]
    services = ['underground', 'metoffice']

The ``internal`` configuration setting allows you to include indoor
temperature and humidity in your uploads.

Note that ``password`` is not the password you use to log in to Weather
Underground, it's the ``Key`` value shown on your list of devices:
https://www.wunderground.com/member/devices

Previous versions of pywws had an extra ``underground_rf`` service to
use Weather Underground's "rapid fire" server for frequent uploads. Now
the rapid fire server is used automatically for "live" data and the
normal server for past data.

.. _Weather Underground: http://www.wunderground.com/

"""

from __future__ import absolute_import, unicode_literals

from ast import literal_eval
from contextlib import contextmanager
from datetime import datetime, timedelta
import logging
import os
import sys

import requests

import pywws.service

__docformat__ = "restructuredtext en"
service_name = os.path.splitext(os.path.basename(__file__))[0]
logger = logging.getLogger(__name__)
RTFREQ = timedelta(seconds=48)


class ToService(pywws.service.CatchupDataService):
    config = {
        'station' : ('',      True, 'ID'),
        'password': ('',      True, 'PASSWORD'),
        'internal': ('False', True, None),
        }
    fixed_data = {'action': 'updateraw', 'softwaretype': 'pywws'}
    interval = timedelta(seconds=47)
    logger = logger
    service_name = service_name
    template = """
#live#
#idx          "'dateutc'     : '%Y-%m-%d %H:%M:%S',"#
#wind_dir     "'winddir'     : '%.0f'," "" "winddir_degrees(x)"#
#wind_ave     "'windspeedmph': '%.2f'," "" "wind_mph(x)"#
#wind_gust    "'windgustmph' : '%.2f'," "" "wind_mph(x)"#
#hum_out      "'humidity'    : '%.d',"#
#temp_out     "'tempf'       : '%.1f'," "" "temp_f(x)"#
#rel_pressure "'baromin'     : '%.4f'," "" "pressure_inhg(x)"#
#calc "temp_f(dew_point(data['temp_out'], data['hum_out']))" "'dewptf': '%.1f',"#
#calc "rain_inch(rain_hour(data))" "'rainin': '%g',"#
#calc "rain_inch(rain_day(data))" "'dailyrainin': '%g',"#
"""

    def __init__(self, context, check_params=True):
        super(ToService, self).__init__(context, check_params)
        # extend template
        if context.params.get('config', 'ws type') == '3080':
            self.template += """
#illuminance  "'solarradiation': '%.2f'," "" "illuminance_wm2(x)"#
#uv           "'UV'            : '%d',"#
"""
        if literal_eval(self.params['internal']):
            self.template += """
#hum_in       "'indoorhumidity': '%.d',"#
#temp_in      "'indoortempf'   : '%.1f'," "" "temp_f(x)"#
"""

    @contextmanager
    def session(self):
        with requests.Session() as session:
            yield session, 'OK'

    def upload_data(self, session, prepared_data={}):
        # extract timestamp from prepared_data
        idx = datetime.strptime(prepared_data['dateutc'], '%Y-%m-%d %H:%M:%S')
        # use "rapid fire" server if data is current
        if datetime.utcnow() - idx < RTFREQ:
            prepared_data.update({'realtime': '1', 'rtfreq': '48'})
            url = 'https://rtupdate.wunderground.com/'
        else:
            url = 'https://weatherstation.wunderground.com/'
        url += 'weatherstation/updateweatherstation.php'
        try:
            rsp = session.get(url, params=prepared_data, timeout=60)
        except Exception as ex:
            return False, repr(ex)
        if rsp.status_code != 200:
            return False, 'http status: {:d}'.format(rsp.status_code)
        text = rsp.text.strip()
        return text == 'success', 'server response "{:s}"'.format(text)


if __name__ == "__main__":
    sys.exit(pywws.service.main(ToService))



# pywws - Python software for USB Wireless Weather Stations
# http://github.com/jim-easterbrook/pywws
# Copyright (C) 2008-24  pywws contributors

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

"""Post messages to Bluesky.

Bluesky is a micro-blogging system that is a whole lot better than the
cesspool Twitter has become under Space Karen. This module
sends "skeets", with optional image files, typically to report on weather
conditions every hour.


* Create account: https://bsky.app/
* Example ``weather.ini`` configuration::

    [bluesky]
    handle = bishopstonwthr.bsky.social
    password = xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

    [hourly]
    text = ['skeet.txt', '24hrs.txt']
    plot = ['24hrs.png.xml', 'rose_12hrs.png.xml']
    services = [('bluesky', 'skeet.txt'), ('ftp', '24hrs.png', 'rose_12hrs.png')]

Create an account
-----------------

You could post weather updates to your regular Bluesky but it may be 
cleaner to have a separate account just for weather reports.

The :py:mod:`pywws.service.bluesky` module requires you to install an
additional dependency::

    sudo pip install atprototools

Authorise pywws
---------------

Before you can send "skeets" you need to authorise pywws to post to your
account. If you run pywws on a low power device such as a Raspberry Pi,
you may find it easier to generate an app password on a browser on
another computer and type the values into the ``weather.ini`` file using
any text editor.

Log into your Bluesky account in a browser and navigate to Settings ->
Privacy and security -> App Passwords -> Add App Password. Give it a 
memorable name (or use a randomly generated one) and then make a note of
the password in the next section; it won't be displayed again. This 
password will give access to your account, and should be kept
confidential.

Create a template
-----------------

A "skeet" is a short text of up to 300 characters. It's up to you what to
put in your "skeet" but an example is included to get your started. Copy
the example template ``skeet.txt`` to your template directory, then edit
it to suit your preferences. (Note hashtags are not automatically
rendered as hash tags, and getting them to work is complicated as you
have to figure out the start/end position in the post text, that's just
too hard for this currently). You should also check it uses the same text
encoding as your other templates. The example template includes a
``media`` line to send a graph. Either remove this or copy the example
graph template ``tweet.png.xml`` to your graph templates directory, if
you don't already have one there.

Now generate a skeet file from your template, for example::

    python -m pywws.template ~/weather/data ~/weather/templates/skeet.txt skeet.txt
    cat skeet.txt

Post your first skeet
--------------------

Now you are ready to run :py:mod:`pywws.service.bluesky`::

    python -m pywws.service.bluesky ~/weather/data skeet.txt

If this works, your new Bluesky account will have posted its first
weather report. (You can delete the skeet.txt file now.)

Add Bluesky posts to your hourly tasks
---------------------------------------

Edit the ``[hourly]`` section in ``weather.ini``. If your skeets include
one or more graphs you need to add the graph templates to the ``plot``
list. Note that if you reuse your Twitter graph you only need to
generate it once. Add your skeet template to the ``text`` list. Finally,
add ``bluesky`` to the ``services`` list, with an option specifying the
template processing result. For example::

    [hourly]
    text = ['skeet.txt']
    plot = ['tweet.png.xml']
    services = [('bluesky', 'skeet.txt')]

You could use the ``[logged]``, ``[12 hourly]`` or ``[daily]`` sections
instead, but I think ``[hourly]`` is most appropriate for Bluesky
updates.

Include images in your skeet
----------------------------

Each post contains up to four images, and each image can have its own
alt text and is limited to 1,000,000 bytes in size. However, the
atprototools libary used to interface with Bluesky is currently coded
to just manage a single image; it also cannot accept ALT text for the
image.

To include an image ensure the first line of the skeet is ``media path``
where ``path`` is the file name or full path for file
that are not in your "output" directory (a subdirectory of your work
directory called ``output``). . The "skeet_media.txt" example template
shows how to do this.

The image could be from a web cam, or for a weather forecast it could be
an icon representing the forecast. To add a weather graph you need to
make sure the graph is drawn before the tweet is sent. The
:py:mod:`pywws.regulartasks` module processes graph and text templates
before doing service uploads, so you can include the graph drawing in
the same section. For example::

    [hourly]
    plot = ['skeet.png.xml']
    text = ['skeet_media.txt']
    services = [('bluesky', 'skeet_media.txt')]

.. _Bluesky: https://bsky.app/

"""

from __future__ import absolute_import, print_function

import codecs
from contextlib import contextmanager
import logging
import os
import sys

import atprototools

import pywws.service

__docformat__ = "restructuredtext en"
service_name = os.path.splitext(os.path.basename(__file__))[0]
logger = logging.getLogger(__name__)

class ToService(pywws.service.FileService):
    config = {
        'handle'      : (None, True, None),
        'password'    : (None, True, None),
        }
    logger = logger
    service_name = service_name

    def __init__(self, context, check_params=True):
        super(ToService, self).__init__(context, check_params)
        # get default character encoding of template output
        self.encoding = context.params.get(
            'config', 'template encoding', 'iso-8859-1')

    @contextmanager
    def session(self):
        yield atprototools.Session(self.params['handle'], self.params['password'])

    def upload_file(self, session, filename):
        media=None
        with codecs.open(filename, 'r', encoding=None) as skeet_file:
            skeet = skeet_file.read()
        while skeet.startswith('media'):
            media_item, skeet = skeet.split('\n', 1)
            media_item = media_item.split()[1]
            if not os.path.isabs(media_item):
                media = os.path.join(self.context.output_dir, media_item)
        try:
            session.postBloot(skeet,image_path=media)
        except Exception as ex:
            return False, repr(ex)
        return True, 'OK'

    def register(self):
        self.check_params('handle', 'password')
        #not sure what else to put here, or how to get an app password programmatically

if __name__ == "__main__":
    sys.exit(pywws.service.main(ToService))



# pywws - Python software for USB Wireless Weather Stations
# http://github.com/jim-easterbrook/pywws
# Copyright (C) 2008-24  pywws contributors

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

from __future__ import with_statement

from datetime import date
import os
from setuptools import setup
from setuptools import __version__ as setuptools_version
import sys

# read current version info without importing pywws package
if sys.version_info[0] >= 3:
    with open('src/pywws/__init__.py') as f:
        exec(f.read())
else:
    execfile('src/pywws/__init__.py')

# get GitHub repo information
# requires GitPython - 'pip install --user gitpython'
try:
    import git
except ImportError:
    git = None
if git:
    try:
        repo = git.Repo()
        if repo.is_dirty():
            latest = 0
            last_release = None
            for tag in repo.tags:
                if tag.tag.tagged_date > latest:
                    latest = tag.tag.tagged_date
                    last_release = str(tag)
            last_commit = str(repo.head.commit)[:7]
            # regenerate version info, if required
            if last_commit != _commit:
                _release = str(int(_release) + 1)
                _commit = last_commit
            if last_release:
                major, minor, patch = map(int, last_release.split('.'))
                today = date.today()
                year = today.year % 100
                if year == major and today.month == minor:
                    patch += 1
                else:
                    patch = 0
                __version__ = '{:d}.{:d}.{:d}'.format(year, today.month, patch)
            with open('src/pywws/__init__.py', 'r') as vf:
                old_init_str = vf.read()
            new_init_str = "__version__ = '" + __version__ + "'\n"
            new_init_str += "_release = '" + _release + "'\n"
            new_init_str += "_commit = '" + _commit + "'\n"
            if new_init_str != old_init_str:
                with open('src/pywws/__init__.py', 'w') as vf:
                    vf.write(new_init_str)
    except (git.exc.InvalidGitRepositoryError, git.exc.GitCommandNotFound):
        pass

cmdclass = {}
command_options = {}

if 'LANG' in os.environ:
    lang = os.environ['LANG'].split('_')[0]
else:
    lang = 'en'

# add commands to create & build translation files
# requires Babel to be installed
command_options['compile_catalog'] = {
    'domain'    : ('setup.py', 'pywws'),
    'directory' : ('setup.py', 'src/pywws/lang'),
    'use_fuzzy' : ('setup.py', '1'),
    }
command_options['extract_messages'] = {
    'input_dirs'         : ('setup.py', 'src/pywws'),
    'output_file'        : ('setup.py', 'build/gettext/pywws.pot'),
    'no_wrap'            : ('setup.py', '1'),
    'sort_by_file'       : ('setup.py', '1'),
    'add_comments'       : ('setup.py', 'TX_NOTE'),
    'strip_comments'     : ('setup.py', '1'),
    'copyright_holder'   : ('setup.py', 'Jim Easterbrook'),
    'msgid_bugs_address' : ('setup.py', 'jim@jim-easterbrook.me.uk'),
    }
command_options['init_catalog'] = {
    'domain'     : ('setup.py', 'pywws'),
    'input_file' : ('setup.py', 'build/gettext/pywws.pot'),
    'output_dir' : ('setup.py', 'src/pywws/lang'),
    'no_wrap'    : ('setup.py', '1'),
    }
command_options['update_catalog'] = {
    'domain'     : ('setup.py', 'pywws'),
    'input_file' : ('setup.py', 'build/gettext/pywws.pot'),
    'output_dir' : ('setup.py', 'src/pywws/lang'),
    'no_wrap'    : ('setup.py', '1'),
    }

# if sphinx is installed, add commands to build documentation
try:
    from sphinx.setup_command import BuildDoc
    # compile documentation to html
    cmdclass['build_sphinx'] = BuildDoc
    command_options['build_sphinx'] = {
        'source_dir' : ('setup.py', 'src/doc'),
        'build_dir'  : ('setup.py', 'doc'),
        'builder'    : ('setup.py', 'html'),
        }
    # extract strings for translation
    class extract_messages_doc(BuildDoc):
        description = 'extract localizable strings from the documentation'
    cmdclass['extract_messages_doc'] = extract_messages_doc
    command_options['extract_messages_doc'] = {
        'source_dir' : ('setup.py', 'src/doc'),
        'build_dir'  : ('setup.py', 'build'),
        'builder'    : ('setup.py', 'gettext'),
        }
except ImportError:
    pass

# set options for uploading documentation to PyPI
command_options['upload_docs'] = {
    'upload_dir' : ('setup.py', 'doc'),
    }

# set options for building distributions
command_options['sdist'] = {
    'formats'        : ('setup.py', 'gztar'),
    }


setup_kwds = {
    'cmdclass': cmdclass,
    'command_options': command_options,
    }


if tuple(map(int, setuptools_version.split('.')[:2])) < (61, 0):
    from setuptools import find_packages
    # get metadata from pyproject.toml
    import toml
    metadata = toml.load('pyproject.toml')
    with open(metadata['project']['readme']) as ldf:
        long_description = ldf.read()
    find_args = metadata['tool']['setuptools']['packages']['find']
    find_args['where'] = find_args['where'][0]
    packages = find_packages(**find_args)
    # add to setup arguments
    setup_kwds.update(
        version = __version__,
        name = metadata['project']['name'],
        author = metadata['project']['authors'][0]['name'],
        author_email = metadata['project']['authors'][0]['email'],
        url = metadata['project']['urls']['Homepage'],
        description = metadata['project']['description'],
        long_description = long_description,
        classifiers = metadata['project']['classifiers'],
        license = metadata['project']['license']['text'],
        packages = packages,
        package_dir = {'' : 'src'},
        package_data = metadata['tool']['setuptools']['package-data'],
        entry_points = {
            'console_scripts' : [
                '{} = {}'.format(k, v)
                for k, v in metadata['project']['scripts'].items()],
            },
        install_requires = metadata['project']['dependencies'],
        extras_require = metadata['project']['optional-dependencies'],
        zip_safe = metadata['tool']['setuptools']['zip-safe'],
        )

setup(**setup_kwds)
